from distutils.core import setup

setup(
    name='bayes',
    description='Bayes classification implementation',
    author='George Courtsunis',
    author_email='gjcourt@gmail.com',
    url='http://github.com/gjcourt/bayes',
    version='0.1',
    packages=['bayes']
    )

